/* $Header: /home/nygren/courses/6.866/project/src/RCS/apputils.c,v 1.3 1996/03/03 21:07:48 nygren Exp $ */
/*
 * FILE:    apputils.c
 * AUTHOR:  Erik Nygren.
 * PURPOSE: Application utility routines.
 *          Written for 6.866 term project, Fall 1995.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include "image.h"
#include "imagedisplayer.h"
#include "projxform.h"
#include "apputils.h"
#include "util.h"


/* imageinfo_create --
 *    Creates a new imageinfo object.
 */
ImageInfo *
imageinfo_create(Image *im, char *name)
{
  ImageInfo    *imi;
  static int    imi_numctr = 0;	/* Counter for unique image id number */

  if (!im || !name || !*name)
    { fprintf(stderr, "imageinfo_create: invalid args\n");
      return(0); }

  imi = (ImageInfo*)xcalloc("imageinfo_create", 1, sizeof(ImageInfo));
  imi->name = (char*)xcalloc("imageinfo_create", 256, sizeof(char));
  imi->next = NULL;
  imi->num = ++imi_numctr;
  strncpy(imi->name, name, 256);
  imi->im = im;
  imi->active = 1;
  imi->links = NULL;
  
  return(imi);
}

ImageInfo *
imageinfo_create_fromfilename(char *filename)
{
  int    fd;
  Image *im;

  if (-1 == (fd = open(filename, O_RDONLY)))
    { perror("imageinfo_create_fromfilename");
      return(NULL); }
  im = image_loadpnm(fd, filename);
  close(fd);
  if (!im)
    return(NULL);
  
  return(imageinfo_create(im, filename));
}


/* imageinfo_create --
 *    Creates a new imageinfolist object.
 */
ImageInfoList *
imageinfolist_create(void)
{
  ImageInfoList    *imil;

  imil = (ImageInfoList*)xcalloc("imageinfolist_create", 1, 
				 sizeof(ImageInfoList));  
  imil->head = NULL;
  return(imil);
}

/* imageinfolist_get_bynum --
 *     Finds an imageinfo structure by its number.
 *     Returns NULL if not found.
 */
ImageInfo *
imageinfo_get_bynum(ImageInfoList *imil, int num)
{
  ImageInfo *pCur;
  
  for (pCur = imil->head; pCur; pCur = pCur->next)
    if (pCur->num == num)
      return(pCur);
  return(NULL);
}

/* imageinfolist_get_byname --
 *     Finds an imageinfo structure by its name.
 *     Returns NULL if not found.
 */
ImageInfo *
imageinfo_get_byname(ImageInfoList *imil, char *name)
{
  ImageInfo *pCur;
  
  for (pCur = imil->head; pCur; pCur = pCur->next)
    {
      if (0 == strcmp(pCur->name, name))
	return(pCur);
    }
  return(NULL);
}

/* imageinfolist_get_byimage --
 *     Finds an imageinfo structure by its image.
 *     Returns NULL if not found.
 */
ImageInfo *
imageinfo_get_byimage(ImageInfoList *imil, Image *im)
{
  ImageInfo *pCur;
  
  for (pCur = imil->head; pCur; pCur = pCur->next)
    {
      if (im == pCur->im)
	return(pCur);
    }
  return(NULL);
}

/* imageinfolist_addtolist -- 
 *     Adds an imageinfo record to the imageinfolist.
 *     Returns 0 if fails, else returns 1.
 */
int
imageinfolist_addtolist(ImageInfoList *imil, ImageInfo *imi)
{
  if (!imil || !imi)
    { fprintf(stderr, "imageinfolist_addtolist: invalid args\n");
      return(0); }

  if ((NULL != imageinfo_get_byimage(imil, imi->im)) 
      || (NULL != imageinfo_get_byname(imil, imi->name)))
    { fprintf(stderr, "imageinfolist_addtolist: %s already exists\n", 
	      imi->name);
      return(0); }
    
  imi->next = imil->head;
  imil->head = imi;

  return(1);
}

/* imageinfolist_print ---
 *    Prints out the images names, numbers, and links to stdout.
 */
void
imageinfolist_print(ImageInfoList *imil)
{
  ImageInfo     *imiCtr;
  ImageLinkage  *ilCtr;

  printf("\nAvailable images:\n");
  for (imiCtr = imil->head; imiCtr; imiCtr = imiCtr->next)
    {
      printf("  [%d]\t%s", imiCtr->num, imiCtr->name);
      if (imiCtr->links)
	{
	  printf(" (links to");
	  for (ilCtr = imiCtr->links; ilCtr; ilCtr = ilCtr->next)
	    printf(" %d", ilCtr->toimage->num);
	  printf(")\n");
	}
      else
	printf(" (no links)\n");
    }
  printf("\n");
}

/* imageinfo_promptforimage ---
 *    Prompts the user to enter the name of an image and returns the imageinfo.
 */
ImageInfo *
imageinfo_promptforimage(ImageInfoList *allimages, char *prompt)
{
  char       buf[256];
  ImageInfo *imi;

  while (1)
    {
      printf("%s [l for list, c for cancel] > ", prompt);
      scanf("%s",buf);
      if ((0 == strcmp(buf, "l"))
	   || (0 == strcmp(buf, "?")))
	{
	  imageinfolist_print(allimages);
	  continue;
	}
      if ((0 == strcmp(buf, "q"))
	   || (0 == strcmp(buf, "c")))
	{
	  return(NULL);
	}
      if (isdigit(*buf) &&
	  (NULL != (imi = imageinfo_get_bynum(allimages, atoi(buf)))))
	break;
      if (NULL != (imi = imageinfo_get_byname(allimages, buf)))
	break;
      printf("No such image %s\n", buf);
    }
  return(imi);
}

/* imageinfo_findlink -- 
 *     Finds a link from imfrom to imto.
 *     Returns the ImageLinkage or NULL if not found.
 */
ImageLinkage *
imageinfo_findlink(ImageInfo *imfrom, ImageInfo *imto)
{
  ImageLinkage *pCur;
  
  if (!imfrom || !imto)
    return(NULL);
  for (pCur = imfrom->links; pCur; pCur = pCur->next)
    if (pCur->toimage == imto)
      return(pCur);
  return(NULL);
}

/* imagelinkage_create -- 
 *    Creates a new image linkage.
 */
ImageLinkage *
imagelinkage_create(ImageInfo *toim, ProjXForm *xform)
{
  ImageLinkage    *il;

  il = (ImageLinkage*)xcalloc("imagelinkage_create", 1, 
			      sizeof(ImageLinkage));  
  il->next = NULL;
  il->toimage = toim;
  il->xform = xform;

  return(il);
}

/* imagelinkage_destroy --
 *    Destroys the linkage and the transform between two images.
 */
void
imagelinkage_destroy(ImageLinkage *iml)
{
  projxform_destroy(iml->xform);
  xfree(iml);
}

/* imageinfo_deletelink -- 
 *     Delete the links images im0 and im1 have between each other.
 *     If no links exist, just returns.
 */
void
imageinfo_deletelink(ImageInfo *im0, ImageInfo *im1)
{
  ImageLinkage *pCur, *pSave;

  if (im0->links && (im0->links->toimage == im1))
    {
      pSave = im0->links;      
      im0->links = im0->links->next;
      imagelinkage_destroy(pSave);
    }
  else
    {
      for (pCur = im0->links; pCur && pCur->next; pCur = pCur->next)
	if (pCur->next->toimage == im1)
	  {
	    pSave = pCur->next;
	    pCur->next = pCur->next->next;
	    imagelinkage_destroy(pSave);
	    break;
	  }
    }

  /* Now delete the other way around */
  if (im1->links && (im1->links->toimage == im0))
    {
      pSave = im1->links;      
      im1->links = im1->links->next;
      imagelinkage_destroy(pSave);
    }
  else
    {
      for (pCur = im1->links; pCur && pCur->next; pCur = pCur->next)
	if (pCur->next->toimage == im0)
	  {
	    pSave = pCur->next;
	    pCur->next = pCur->next->next;
	    imagelinkage_destroy(pSave);
	    break;
	  }
    }
}


/* imageinfo_makelinkage --
 *    Generates a linkage between two images by prompting the
 *    user to select numpts points.
 *    Deletes any existing linkages.
 *    Returns 0 on failure, else 1.
 */
int
imageinfo_makelinkage(ImageInfo *imi0, ImageInfo *imi1, int numpts)
{
  ImageDisplayer *id0, *id1;
  Image          *im0, *im1;
  int             ptctr;
  PointPairList  *ppl;
  ProjXForm      *pxf0, *pxf1;
  ImageLinkage   *il0, *il1;

  if (!imi0 || !imi1)
    { fprintf(stderr, "imageinfo_makelinkage: bad args\n");
      return(0);  }
  if (numpts < 5)
    { fprintf(stderr,
	   "imageinfo_makelinkage: the number of points must be 5 or more\n");
      return(0);  }

  /* Check to see if the linkage exists and act accordingly. */
  if ((NULL != imageinfo_findlink(imi0, imi1))
      || (NULL != imageinfo_findlink(imi1, imi0)))
    {
      char inbuf[256];

      printf("A linkage already exists between images %s and %s.\n",
	     imi0->name, imi1->name);
      printf("Do you wish to replace this? (y/n) > ");
      scanf("%s", inbuf);
      if ((0 != strcmp(inbuf, "yes"))
	   && (0 != strcmp(inbuf, "y"))
	   && (0 != strcmp(inbuf, "YES"))
	   && (0 != strcmp(inbuf, "Y")))
	{
	  printf("Ok.  Linkage not deleted.\n");
	  return(0);
	}
      imageinfo_deletelink(imi0, imi1);
    }

  im0 = imi0->im;
  im1 = imi1->im;

  id0 = imagedisplayer_create(im0->xres, im0->yres, imi0->name);
  id1 = imagedisplayer_create(im1->xres, im1->yres, imi1->name);
  if (!id0 || !id1)
    { fprintf(stderr, "imageinfo_makelinkage: error opening window\n");
      return(0); }
  
  imagedisplayer_putimage(id0, im0);
  imagedisplayer_putimage(id1, im1);
  
  ppl = pointpairlist_create();
  
  printf("\n");
  printf("You will now be asked to select %d corresponding points\n", numpts);
  printf("between the two images.  For best results, they should be\n");
  printf("widely dispersed and shouldn't be concentrated in a region\n");
  printf("or on a line.  You may start now...\n");

  for (ptctr=1; ptctr<=numpts; ptctr++)
    {
      int ax, ay, bx, by;
      
      printf("\n(%d/%d): Select a point in Window 0: ", ptctr, numpts);
      imagedisplayer_get_buttonpress(id0, &ax, &ay);
      ax -= (im0->xres)/2;
      ay -= (im0->yres)/2;
      printf("(%d %d)\n", ax, ay);
      printf("(%d/%d): Select a point in Window 1: ", ptctr, numpts);
      imagedisplayer_get_buttonpress(id1, &bx, &by);
      bx -= (im0->xres)/2;
      by -= (im0->yres)/2;
      printf("(%d %d)\n", bx, by);
      pointpairlist_addpair(ppl, ax, ay, bx, by);
    }

  pxf0 = pointpairlist_genprojxform(ppl);  /* im0=A, im1=B */
  ppl = pointpairlist_flipab(ppl);
  pxf1 = pointpairlist_genprojxform(ppl);  /* im0=B, im1=A */

  imagedisplayer_destroy(id0);
  imagedisplayer_destroy(id1);

  if (!pxf0 || !pxf1)
    { fprintf(stderr,"imageinfo_makelinkage: failed to find transform,\n");
      fprintf(stderr,"\tpossibly due to a singularity.  Try more points.\n");
      return(0);
    }

  il0 = imagelinkage_create(imi1, pxf0);
  il1 = imagelinkage_create(imi0, pxf1);
  
  /* Add the linkages to the images. */
  il0->next = imi0->links;
  imi0->links = il0;
  il1->next = imi1->links;
  imi1->links = il1;

  return(1);
}

/* imageinfolist_untouchall ---
 *    Clears the touched flag for all imageinfo objects in the imageinfolist.
 */
void
imageinfolist_untouchall(ImageInfoList *imil)
{
  ImageInfo *imiCur;  

  /* Untouch all images... */
  for (imiCur = imil->head; imiCur; imiCur = imiCur->next)
    imiCur->touched = 0;
}

/* imageinfo_rendertree ---
 *    Renders a tree of linked images.  Creates a new image to
 *    store the rendered image into.  Centerimi is
 *    imageinfo object for the image which should be
 *    at the center of the rendering.
 */
Image *
imageinfo_rendertree(ImageInfoList *allimages,
		 ImageInfo     *centerimi,
		 int xres, int yres, int np, int quick)
{
  Image      *destim;
  ProjXForm  *basexform;	/* Start from this transform */

  imageinfolist_untouchall(allimages);
  destim = image_create(xres, yres, np);
  destim->xorigin = xres/2;
  destim->yorigin = yres/2;

  basexform = projxform_create_identity();

  imageinfo_render_recurs(destim, centerimi, basexform, quick);

  projxform_destroy(basexform);
  return(destim);
}


/* imageinfo_render_recurs ---
 *    Renders the image curimi then recurses to render all images
 *    linked to it.  Sets the touched flag for curimi.
 */
void
imageinfo_render_recurs(Image *destim, ImageInfo *curimi,
			ProjXForm *curxform, int quick)
{
  ProjXForm *childxform;
  ImageInfo *childimi;
  ImageLinkage *pCurLink;

  /* Don't render an image twice */
  if (curimi->touched)
    return;
  curimi->touched = 1;

#undef RENDERTOPDOWN
#ifdef RENDERTOPDOWN
  printf("Rendering %s...\n", curimi->name);
  if (quick)
    projxform_xform_image_quick(curxform, destim, curimi->im);
  else
    projxform_xform_image(curxform, destim, curimi->im);
#endif

  for (pCurLink = curimi->links; pCurLink; pCurLink = pCurLink->next)
    if (!(pCurLink->toimage->touched))  /* Don't worry with touched images */
    {
      childimi = pCurLink->toimage;
      childxform = projxform_compose(curxform, pCurLink->xform);
      imageinfo_render_recurs(destim, childimi, childxform, quick);
      projxform_destroy(childxform);
    }

#ifndef RENDERTOPDOWN
  printf("Rendering %s...\n", curimi->name);
  if (quick)
    projxform_xform_image_quick(curxform, destim, curimi->im);
  else
    projxform_xform_image(curxform, destim, curimi->im);
#endif

}


/* imageinfolist_saveall ---
 *    Saves all the information about images and linkages
 *    between images.  This is meant to be a temporary
 *    format (it's binary and byte order specific and
 *    stores absolute image pathnames the same as when they
 *    were loaded).
 *
 *  File Format (binary)
 *
 *       Field Size        Contents
 *       ----------        --------
 *       8*char            Magic number "PXF0000\0"
 *
 *    For each image:
 *       1*int             Image number
 *       256*char          Image filename
 *       
 *       1*int             Integer 0 (image list terminator)
 *
 *    For each projective transform:
 *       1*int             Image from
 *       1*int             Image to
 *       10*double         ProjXForm params (D,E,F,K,L,M,Gx,Hx,Gy,Hy)
 *
 *       2*int             Integer 0,0 (transform list terminator)
 */
int
imageinfolist_saveall(ImageInfoList *imil, char *filename)
{
  int fd;
  ImageInfo    *imiCur;
  ImageLinkage *linkCur;
  char          charbuf[256];
  double        doublebuf[10];
  int           intbuf[2];

  if (-1 == (fd = open(filename, O_WRONLY | O_CREAT | O_TRUNC, 0644)))
    { fprintf(stderr, "imageinfolist_saveall: couldn't open save file\n");
      return(0); }

  /* Store magic number */
  write(fd, SAVEFILE_MAGIC, 8*sizeof(char));
  
  /* Store image names and numbers */
  for (imiCur = imil->head; imiCur; imiCur = imiCur->next)
    {
      intbuf[0] = imiCur->num;
      write(fd, intbuf, sizeof(int));
      strncpy(charbuf, imiCur->name, 256);
      write(fd, charbuf, 256*sizeof(char));
    }
  intbuf[0] = 0;
  write(fd, intbuf, sizeof(int));

  for (imiCur = imil->head; imiCur; imiCur = imiCur->next)
    for (linkCur = imiCur->links; linkCur; linkCur = linkCur->next)
      {
	intbuf[0] = imiCur->num;
	intbuf[1] = linkCur->toimage->num;
	write(fd, intbuf, 2*sizeof(int));
	doublebuf[0] = linkCur->xform->D;
	doublebuf[1] = linkCur->xform->E;
	doublebuf[2] = linkCur->xform->F;
	doublebuf[3] = linkCur->xform->K;
	doublebuf[4] = linkCur->xform->L;
	doublebuf[5] = linkCur->xform->M;
	doublebuf[6] = linkCur->xform->Gx;
	doublebuf[7] = linkCur->xform->Hx;
	doublebuf[8] = linkCur->xform->Gy;
	doublebuf[9] = linkCur->xform->Hy;
	write(fd, doublebuf, 10*sizeof(double));
      }
  intbuf[0] = 0;
  intbuf[1] = 0;
  write(fd, intbuf, 2*sizeof(int));
  close(fd);

  return(1);
}


#define MAXIMINUM 512  /* The maximum value for ImageInfo->num */
#define READNTEST(fd,buf,size) \
        if (0>=read((fd),(buf),(size))) \
		    { fprintf(stderr, "imageinfolist_load: read error\n"); \
			return(0); }
  
/* imageinfolist_load ---
 *    Loads a file containing image names and transforms.
 *    Remaps image numbers.  Returns 0 on error else 1.
 */
int
imageinfolist_load(ImageInfoList *imil, char *filename)
{
  int nummap[MAXIMINUM+1];  /* Mapping of image number
			       * in files to new numbers */
  int fd;
  ImageInfo    *imiCur, *imiFrom, *imiTo;
  ImageLinkage *linkCur;
  char          charbuf[256];
  double        doublebuf[10];
  int           intbuf[2];
  ProjXForm    *pxf;
  int           ctr;

  if (-1 == (fd = open(filename, O_RDONLY)))
    { fprintf(stderr, "imageinfolist_load: couldn't open file\n");
      return(0); }

  for (ctr=0; ctr<=MAXIMINUM; ctr++) nummap[ctr] = 0;

  READNTEST(fd, charbuf, 8*sizeof(char));
  if (0 != strncmp(charbuf, SAVEFILE_MAGIC, 8))
    { fprintf(stderr, "imageinfolist_load: bad magic number %s\n", charbuf);
      close(fd);
      return(0); }
    
  /* Read images until we reach a zero */
  while (1)			
    {
      READNTEST(fd, intbuf, 1*sizeof(int));
      if (intbuf[0] <= 0)
	break;
      READNTEST(fd, charbuf, 256*sizeof(char));
      if (intbuf[0] > MAXIMINUM)
	{ fprintf(stderr,"imageinfolist_load: image number %d out of range\n",
		  intbuf[0]);
	  continue; }
      /* If exists, enter number into mapping.  Otherwise, load. */
      if (NULL != (imiCur=imageinfo_get_byname(imil, charbuf)))
	{
	  printf("Image %s already exists as image number %d\n",
		 imiCur->name, imiCur->num);
	}
      else
	{
	  if (!(imiCur = imageinfo_create_fromfilename(charbuf)))
	    { fprintf(stderr,"imageinfolist_load: couldn't load %s\n",
		      charbuf);
	      continue; }
	  if (0 == imageinfolist_addtolist(imil, imiCur))
	    continue;
	}
      nummap[intbuf[0]] = imiCur->num;
    }

  /* Now load image linkages */
  while (1)
    {
      READNTEST(fd, intbuf, 2*sizeof(int));
      if (intbuf[0] == 0)
	break;
      READNTEST(fd, doublebuf, 10*sizeof(double));

      if (intbuf[0] <= 0 
	  || intbuf[0] >= MAXIMINUM
	  || intbuf[1] <= 0
	  || intbuf[1] >= MAXIMINUM)
	{ fprintf(stderr,"imageinfolist_load: image number out of range\n");
	  continue; }

      if (nummap[intbuf[0]] <= 0 || nummap[intbuf[1]] <= 0)
	{ fprintf(stderr,"imageinfolist_load: invalid mapping\n");
	  continue; }

      imiFrom = imageinfo_get_bynum(imil, nummap[intbuf[0]]);
      imiTo = imageinfo_get_bynum(imil, nummap[intbuf[1]]);
      if (!imiFrom || !imiTo)
	{ fprintf(stderr,"imageinfolist_load: couldn't find image\n");
	  continue; }

      pxf = projxform_create(doublebuf[0], doublebuf[1], doublebuf[2], 
			     doublebuf[3], doublebuf[4], doublebuf[5], 
			     doublebuf[6], doublebuf[7], 
			     doublebuf[8], doublebuf[9]);
      linkCur = imagelinkage_create(imiTo, pxf);
      linkCur->next = imiFrom->links;
      imiFrom->links = linkCur;
    }
  
  close(fd);
  return(1);
}
#undef READNTEST
