/* $Header: /home/nygren/courses/6.866/project/src/RCS/apputils.h,v 1.2 1996/03/03 02:07:22 nygren Exp $ */
/*
 * FILE:    apputils.h
 * AUTHOR:  Erik Nygren.
 * PURPOSE: Header for application utility functions
 *          Written for 6.866 term project, Fall 1995.
 */

#ifndef INC_APPUTILS_H
#define INC_APPUTILS_H

#include "image.h"
#include "projxform.h"

#define SAVEFILE_MAGIC "PXF0000\0" /* Magic number for saved files.
				    * Must be 8 characters long. */

typedef struct _ImageLinkage {
  struct _ImageLinkage *next;	   /* The next in the linked list */
  struct _ImageInfo    *toimage;   /* The image this is a link to */
  ProjXForm            *xform;     /* The transform relating this image
				    * to "toimage".  If this image is "A"
				    * then toimage is "B". */
} ImageLinkage;

typedef struct _ImageInfo {
  struct _ImageInfo  *next;	/* The next in the linked list */
  Image              *im;       /* The image itself */
  int                 num;	/* The identification number (num > 0) */
  char               *name;     /* The name of the image */
  int                 active;	/* Is the image active */
  ImageLinkage       *links;	/* The links between this image and
				 * other images */
  int                 touched;  /* Flag set when rendering to
				 * avoid circularity */
} ImageInfo;

typedef struct _ImageInfoList {
  ImageInfo          *head;
} ImageInfoList;


extern ImageInfo *
imageinfo_create(Image *im, char *name);

extern ImageInfo *
imageinfo_create_fromfilename(char *filename);

extern ImageInfoList *
imageinfolist_create(void);

extern ImageInfo *
imageinfo_promptforimage(ImageInfoList *allimages, char *prompt);

extern void
imageinfolist_print(ImageInfoList *imil);

extern ImageInfo *
imageinfo_get_bynum(ImageInfoList *imil, int num);

extern ImageInfo *
imageinfo_get_byname(ImageInfoList *imil, char *name);

extern ImageInfo *
imageinfo_get_byimage(ImageInfoList *imil, Image *im);

extern int
imageinfolist_addtolist(ImageInfoList *imil, ImageInfo *imi);

extern ImageLinkage *
imageinfo_findlink(ImageInfo *imfrom, ImageInfo *imto);

extern ImageLinkage *
imagelinkage_create(ImageInfo *toim, ProjXForm *xform);

extern void
imagelinkage_destroy(ImageLinkage *iml);

extern void
imageinfo_deletelink(ImageInfo *im0, ImageInfo *im1);

extern int
imageinfo_makelinkage(ImageInfo *imi0, ImageInfo *imi1, int numpts);

extern void
imageinfolist_untouchall(ImageInfoList *imil);

extern Image *
imageinfo_rendertree(ImageInfoList *allimages,
		 ImageInfo     *centerimi,
		 int xres, int yres, int np, int quick);

extern void
imageinfo_render_recurs(Image *destim, ImageInfo *curimi,
			ProjXForm *curxform, int quick);

extern int
imageinfolist_load(ImageInfoList *imil, char *filename);

extern int
imageinfolist_saveall(ImageInfoList *imil, char *filename);


#endif /* INC_APPUTILS_H */
