/* $Header: /home/nygren/courses/6.866/project/src/RCS/image.h,v 1.6 1996/03/02 17:09:59 nygren Exp $ */
/*
 * FILE:    image.h
 * AUTHOR:  Erik Nygren.
 * PURPOSE: Header for image loading routines.
 *          Written for 6.866 term project, Fall 1995.
 */

#ifndef INC_IMAGE_H
#define INC_IMAGE_H

#include "util.h"

#define MAX_IMAGEPLANES 3  /* How many image planes can an image have? */

/* IPlane data structure.  Contains a plane of data for an image.
 * The plane is a xres by yres array of 8 bit pixel values. */
typedef struct __IPlane {
  int    xres, yres;   /* The width and height of the plane  */
  int xorigin, yorigin; /* Where in the image the origin is */
  uchar *data;         /* The image data (size is xres*yres) */
} IPlane;

typedef struct __Image {
  int   xres, yres;   /* The width and height of the image */
  int   np;           /* The number of IPlane's            */
  char *source;       /* Where does this image come from?  */
  int xorigin, yorigin; /* Where in the image the origin is */
  IPlane *p[MAX_IMAGEPLANES];   /* The image planes (3 max for now)  */
} Image;

/* Used for parsing */
typedef struct __Scanner {
  int    fd;
  char   cur;
  int    eof;
} Scanner;

extern Image *
image_create(int xres, int yres, int np);

extern Image *
image_loadpnm(int fd, char *filename);

extern Image *
image_copy(Image *im);

extern void
image_destroy(Image *im);

extern int  
image_writepnm(Image *im, int fd);

extern uchar
iplane_getval(IPlane *ip, int x, int y);

extern uchar
iplane_getval_rel(IPlane *ip, int x, int y);

extern void
image_setval_rel(Image *im, int x, int y, ulong32 val);

extern ulong32
image_getval_rel(Image *im, int x, int y);

extern Scanner *
scanner_create(int fd);

extern void
scanner_gettoken(Scanner *s, char *buf, int bufsize);

extern void
scanner_eatwhitespace(Scanner *s);

#define scanner_eof(s) ((s)->eof)

extern void
imageplane_drawline(IPlane *bm, uchar color, 
		    int xFrom, int yFrom, int xTo, int yTo);

#endif /* INC_IMAGE_H */
