/* $Header: /home/nygren/courses/6.866/project/src/RCS/projxform.h,v 1.5 1996/03/03 02:07:22 nygren Exp $ */
/*
 * FILE:    projxform.h
 * AUTHOR:  Erik Nygren.
 * PURPOSE: Header for projective transform operations.
 *          Written for 6.866 term project, Fall 1995.
 */

#ifndef INC_PROJXFORM_H
#define INC_PROJXFORM_H

#include "image.h"

/* Projective transformation of the form:
 *
 *       D*ax + E*ay + F
 *  bx = ---------------
 *       Gx*ax + Hx*ay + 1
 *
 *       K*ax + L*ay + M
 *  by = ---------------
 *       Gy*ax + Hy*ay + 1
 *
 */

typedef struct _ProjXForm {
  double D, E, F;
  double K, L, M;
  double Gx, Hx;
  double Gy, Hy;
} ProjXForm;

typedef struct _PointPair {
  struct _PointPair *next;	      /* For chaining into a linked list */
  double             ax, ay, bx, by;
} PointPair;

typedef struct _PointPairList {
  PointPair         *head;
  int                numpairs;
} PointPairList;

extern ProjXForm *
projxform_create(double D, double E, double F, double K, double L, double M,
		 double Gx, double Hx, double Gy, double Hy);

extern int
projxform_xform(ProjXForm *pxf,
		double ax, double ay, double *bx, double *by);

extern ProjXForm *
projxform_create_identity(void);

extern ProjXForm *
projxform_compose(ProjXForm *AB, ProjXForm *BC);

extern void
projxform_destroy(ProjXForm *pxf);

extern void
projxform_xform_image(ProjXForm *pxf, Image *imA, Image *imB);

extern void
projxform_xform_image_quick(ProjXForm *pxf, Image *imA, Image *imB);

extern PointPairList *
pointpairlist_create(void);

extern void
pointpairlist_addpair(PointPairList *ppl, double ax, double ay,
		      double bx, double by);

extern PointPairList *
pointpairlist_flipab(PointPairList *pplin);

extern ProjXForm *
pointpairlist_genprojxform(PointPairList *ppl);


#endif /* INC_PROJXFORM_H */
