/* $Header: /home/nygren/courses/6.866/project/src/RCS/testfindproj.c,v 1.3 1996/02/26 02:45:46 nygren Exp $ */
/*
 * FILE:    testfindproj.c
 * AUTHOR:  Erik Nygren.
 * PURPOSE: Tests a bunch of things.
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include "image.h"
#include "imagedisplayer.h"
#include "projxform.h"


/* prototypes */
void getkey(void);

void
getkey(void)
{
  char buf[20];
  printf("press a key...");
  fgets(buf, 19, stdin);
}

void
main(int argc, char *argv[])
{
  Image *im0, *im1, *imMerge;
  ImageDisplayer *id0, *id1, *idMerge;
  int fd;
  int ptctr, numpts;
  PointPairList *ppl;
  ProjXForm *pxf0, *pxf1;

  if (argc<4)
    {
      fprintf(stderr, "Invalid args\n");
      exit(-1);
    }
  
  fd = open(argv[1], O_RDONLY);
  im0 = image_loadpnm(fd, argv[1]);
  close(fd);

  fd = open(argv[2], O_RDONLY);
  im1 = image_loadpnm(fd, argv[2]);
  close(fd);
  
  numpts = atoi(argv[3]);

  if (!im0)
    { fprintf(stderr, "Error loading image %s\n", argv[1]);
      exit(-1); }

  if (!im1)
    { fprintf(stderr, "Error loading image %s\n", argv[2]);
      exit(-1); }

  imagedisplayer_init();
  id0 = imagedisplayer_create(im0->xres, im0->yres, argv[1]);
  id1 = imagedisplayer_create(im1->xres, im1->yres, argv[2]);
  if (!id0 || !id1)
    {
      fprintf(stderr, "Error opening window\n");
      exit(-1);
    }
  
  imagedisplayer_putimage(id0, im0);
  imagedisplayer_putimage(id1, im1);
  
  ppl = pointpairlist_create();
  
  if (numpts > 0)
    {
      for (ptctr=1; ptctr<=numpts; ptctr++)
	{
	  int ax, ay, bx, by;
	  
	  printf("\n(%d/%d): Select a point in Window 0: ", ptctr, numpts);
	  imagedisplayer_get_buttonpress(id0, &ax, &ay);
	  ax -= (im0->xres)/2;
	  ay -= (im0->yres)/2;
	  printf("(%d %d)\n", ax, ay);
	  printf("(%d/%d): Select a point in Window 1: ", ptctr, numpts);
	  imagedisplayer_get_buttonpress(id1, &bx, &by);
	  bx -= (im0->xres)/2;
	  by -= (im0->yres)/2;
	  printf("(%d %d)\n", bx, by);
	  pointpairlist_addpair(ppl, ax, ay, bx, by);
	}
    }
  else
    {
#ifdef BADSET
      pointpairlist_addpair(ppl, 41, -48, -119, -53);
      pointpairlist_addpair(ppl, 40, 54, -120, 51);
      pointpairlist_addpair(ppl, 153, 42, -7, 36);
      pointpairlist_addpair(ppl, 20, 80, -140, 78);
      pointpairlist_addpair(ppl, 98, 52, -59, 47);
      pointpairlist_addpair(ppl, 17, 36, -142, 33);
      pointpairlist_addpair(ppl, 122, -60, -39, -62);
#else
      pointpairlist_addpair(ppl, 37, -47, -122, -54);
      pointpairlist_addpair(ppl, 123, -61, -38, -63);
      pointpairlist_addpair(ppl, 35, 54, -123, 51);
      pointpairlist_addpair(ppl, 52, 42, -105, 38);
      pointpairlist_addpair(ppl, 75, 42, -79, 37);
      pointpairlist_addpair(ppl, 96, 50, -62, 45);
      pointpairlist_addpair(ppl, 127, -9, -30, -13);
      pointpairlist_addpair(ppl, 131, 51, -29, 45);
      pointpairlist_addpair(ppl, 133, 110, -27, 101);
      pointpairlist_addpair(ppl, 2, 7, -157, 2);
      pointpairlist_addpair(ppl, 88, -1, -69, -5);
      pointpairlist_addpair(ppl, 81, 87, -78, 82);
      pointpairlist_addpair(ppl, 96, 112, -62, 106);
      pointpairlist_addpair(ppl, 21, 79, -139, 78);
      pointpairlist_addpair(ppl, 139, 8, -22, 3);
#endif
    }

  printf("\nThanks for the points...  Calculating.\n");

#ifdef SWAPPOINTS
  ppl = pointpairlist_flipab(ppl);
  /* Swapped pxf0 and pxf1 because of flip */
  pxf0 = pointpairlist_genprojxform(ppl);
  pxf1 = projxform_create_identity();
#else
  pxf1 = pointpairlist_genprojxform(ppl);
  pxf0 = projxform_create_identity();
#endif

  imMerge = image_create(im0->xres*2, im0->yres*2, im0->np); 

  fprintf(stderr, "Doing transform of %s ... ", argv[1]);
  projxform_xform_image(pxf0, imMerge, im0);
  fprintf(stderr, "Done.\n");

  fprintf(stderr, "Doing transform of %s ... ", argv[2]);
  projxform_xform_image(pxf1, imMerge, im1);
  fprintf(stderr, "Done.\n");

  idMerge = imagedisplayer_create(imMerge->xres, imMerge->yres, 
				  "Merged Projection"); 
  imagedisplayer_putimage(idMerge, imMerge);

  getkey();
}







