/* $Header: /home/nygren/courses/6.866/project/src/RCS/testimage.c,v 1.5 1996/01/27 22:47:28 nygren Exp $ */
/*
 * FILE:    testimage.c
 * AUTHOR:  Erik Nygren.
 * PURPOSE: Reads an image in, plays with it, then writes it out.
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include "image.h"
#include "imagedisplayer.h"

void filter_dim50(Image *im);
void filter_blurgauss(Image *im);
void filter_blursquare(Image *im);
void filter_sharpen(Image *im);
void filter_funky(Image *im);
void filter_badtv(Image *im);
void filter_invert(Image *im);
void filter_diff1(Image *im);
void filter_diff2(Image *im);

void 
image_convolve(Image *im, double *convmask, int convmasklen, double offset);

typedef struct __Filter {
  char *name;
  void (*proc)(Image*);
} Filter;

Filter filters[] = { 
  { "dim50", filter_dim50 },
  { "blurgauss", filter_blurgauss },
  { "blursquare", filter_blursquare },
  { "sharpen", filter_sharpen },
  { "funky", filter_funky },
  { "badtv", filter_badtv },
  { "invert", filter_invert },
  { "diff1", filter_diff1 },
  { "diff2", filter_diff2 },
  { NULL, NULL } };

void
getkey(void)
{
  char buf[20];
  printf("press a key...");
  fgets(buf, 19, stdin);
}

void
main(int argc, char *argv[])
{
  Image *im;
  ImageDisplayer *id;
  int fd;
  int filtnum, argctr;

  if (argc<3)
    {
      fprintf(stderr, "Invalid args\n");
      exit(-1);
    }
  
  if (0==strcmp(argv[1],"-"))
    fd = 0;
  else
    fd = open(argv[1], O_RDONLY);
  im = image_loadpnm(fd, argv[1]);
  close(fd);

  if (!im)
    {
      fprintf(stderr, "Error loading image\n");
      exit(-1);
    }

  imagedisplayer_init();
  id = imagedisplayer_create(im->xres, im->yres, argv[1]);
  if (!id)
    {
      fprintf(stderr, "Error opening window\n");
      exit(-1);
    }

  for (argctr = 3; argctr < argc; argctr++)
    {
      imagedisplayer_putimage(id, im); getkey();
      for (filtnum=0; filters[filtnum].name; filtnum++)
	if (0==strcmp(argv[argctr],filters[filtnum].name))
	  {
	    fprintf(stderr,"Running filter %s\n", filters[filtnum].name);
	    (filters[filtnum].proc)(im);
	    break;
	  }
      if (!filters[filtnum].name)
	fprintf(stderr, "No such filter %s\n", argv[argctr]);
    }

  if (0==strcmp(argv[2],"-"))
    fd = 1;
  else
    fd = open(argv[2], O_WRONLY);
  fprintf(stderr, "Writing image...\n");
  if (!image_writepnm(im, fd))
    {
      fprintf(stderr, "Error saving image\n");
      exit(-1);
    }
  imagedisplayer_putimage(id, im); getkey();

  while(1)
    {
      int x0, y0, x1, y1;
      scanf("%d %d %d %d", &x0, &y0, &x1, &y1);
      imageplane_drawline(im->p[0], 255, x0, y0, x1, y1);
      imagedisplayer_putimage(id, im);
    }

  fprintf(stderr, "Done.\n");
  close(fd);
}

void
filter_dim50(Image *im)
{
  int plane, x,y;
  uchar *ptr;

  for (plane=0; plane<im->np; plane++)
    {
      ptr = im->p[plane]->data;
	for (y=0; y < im->yres; y++)
	  for (x=0; x < im->xres; x++)
	    {
	      *ptr = (*ptr)/2;
	      ptr++;
	    }
    }
}

void
filter_blurgauss(Image *im)
{
  double convmask[] = {.1,.2,.4,.2,.1};
  int convmasklen = 5;
  double offset = 0;

  image_convolve(im, convmask, convmasklen, offset);
}

void
filter_blursquare(Image *im)
{
  double convmask[] = {.2,.2,.2,.2,.2};
  int convmasklen = 5;
  double offset = 0;

  image_convolve(im, convmask, convmasklen, offset);
}

void
filter_funky(Image *im)
{
  double convmask[] = {.2,.6,.2,0,-.2,-.6,-.2};
  int convmasklen = 6;
  double offset = 0;

  image_convolve(im, convmask, convmasklen, offset);
}

void
filter_badtv(Image *im)
{
  double convmask[] = {-1,1,0,0,0,0,0,0,0,0,0,1};
  int convmasklen = 12;
  double offset = 0;

  image_convolve(im, convmask, convmasklen, offset);
}

void
filter_invert(Image *im)
{
  double convmask[] = {-1};
  int convmasklen = 1;
  double offset = 255;

  image_convolve(im, convmask, convmasklen, offset);
}

void
filter_sharpen(Image *im)
{
  double convmask[] = {-1,1,1};
  int convmasklen = 3;
  double offset = 0;

  image_convolve(im, convmask, convmasklen, offset);
}

void
filter_diff1(Image *im)
{
  double convmask[] = {-1, 1};
  int convmasklen = 2;
  double offset = 128;

  image_convolve(im, convmask, convmasklen, offset);
}

void
filter_diff2(Image *im)
{
  double convmask[] = {1, -1};
  int convmasklen = 2;
  double offset = 128;

  image_convolve(im, convmask, convmasklen, offset);
}

void
image_convolve(Image *im, double *convmask, int convmasklen, double offset)
{
  int plane, x,y, convctr;
  uchar *ptr;
  double accum;

  for (plane=0; plane<im->np; plane++)
    {
      ptr = im->p[plane]->data;
	for (y=0; y < im->yres; y++)
	  {
	    for (x=0; x < (im->xres)-convmasklen; x++)
	      {
		for (accum=0,convctr=0; convctr<convmasklen; convctr++)
		  accum += (double)(*(ptr+convctr)) * convmask[convctr];
		accum += offset;
		if (accum<0.0) accum=0.0;
		if (accum>255.0) accum=255.0;
		*(ptr++) = (uchar)accum;
	      }
	    ptr += convmasklen;
	  }
    }  
}
