/* $Header: /home/nygren/courses/6.866/project/src/RCS/testopflow.c,v 1.2 1996/02/17 19:36:41 nygren Exp $ */
/*
 * FILE:    testimage.c
 * AUTHOR:  Erik Nygren.
 * PURPOSE: Reads an image in, plays with it, then writes it out.
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include "image.h"
#include "opflow.h"
#include "imagedisplayer.h"


/* prototypes */
void opflow_test(ImageDisplayer *id, IPlane *ip0, IPlane *ip1);
void getkey(void);

void
getkey(void)
{
  char buf[20];
  printf("press a key...");
  fgets(buf, 19, stdin);
}

void
main(int argc, char *argv[])
{
  Image *im0, *im1;
  ImageDisplayer *id;
  int fd;

  if (argc<3)
    {
      fprintf(stderr, "Invalid args\n");
      exit(-1);
    }
  
  fd = open(argv[1], O_RDONLY);
  im0 = image_loadpnm(fd, argv[1]);
  close(fd);

  fd = open(argv[2], O_RDONLY);
  im1 = image_loadpnm(fd, argv[2]);
  close(fd);
  
  if (!im0)
    { fprintf(stderr, "Error loading image %s\n", argv[1]);
      exit(-1); }

  if (!im1)
    { fprintf(stderr, "Error loading image %s\n", argv[2]);
      exit(-1); }

  if ((im0->xres != im1->xres)
      || (im0->yres != im1->yres))
    { fprintf(stderr, "Image resolutions do not match!\n");
      exit(-1); }

  imagedisplayer_init();
  id = imagedisplayer_create(im0->xres, im0->yres, argv[1]);
  if (!id)
    {
      fprintf(stderr, "Error opening window\n");
      exit(-1);
    }

  opflow_test(id, im0->p[0], im1->p[0]);

  fprintf(stderr, "Done.\n");
}


void
opflow_test(ImageDisplayer *id, IPlane *ip0, IPlane *ip1)
{
  DerivField *Ex, *Ey, *Et;
  OpFlow *newOF, *oldOF;

  if ((ip0->xres != ip1->xres)
      || (ip0->yres != ip1->yres))
    {
      fprintf(stderr,"opflow_test: resolutions don't match!\n");
      return;
    }
      
  Ex = derivfield_make_Ex(ip0, ip1);  
  Ey = derivfield_make_Ey(ip0, ip1);  
  Et = derivfield_make_Et(ip0, ip1);  
  
  oldOF = opflow_make(ip0->xres-1, ip0->yres-1);
  
  while (1)
    {
      imagedisplayer_putiplane_withflow(id, ip0, oldOF); getkey();

      newOF = opflow_doiter(oldOF, Ex, Ey, Et, 0.4);
      opflow_destroy(oldOF);
      oldOF = newOF;
    }
}







