/* $Header: /home/nygren/courses/6.866/project/src/RCS/xformtool.c,v 1.2 1996/03/03 02:07:22 nygren Exp $ */
/*
 * FILE:    main.c
 * AUTHOR:  Erik Nygren.
 * PURPOSE: Mainline for 6.866 final project.  Contains 
 *          user interface and driver code.  Eventually,
 *          this could be replaced with a Motif version.
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include "image.h"
#include "imagedisplayer.h"
#include "projxform.h"
#include "apputils.h"

/* Prototypes for menu commands */
void menusel_unimplemented(int flags);
void menusel_createlink(int flags);
void menusel_deletelink(int flags);
void menusel_addimage(int flags);
void menusel_render(int flags);
void menusel_showimage(int flags);
void menusel_listimages(int flags);
void menusel_quit(int flags);
void menusel_load(int flags);
void menusel_save(int flags);

/* Menu command flags */
#define RENDER_TOFILE  0x0001
#define RENDER_QUICK   0x0002

/* Used throughout */
static ImageInfoList *allimages;

static struct menuitem {
  char *abbrev;
  char *itemname;
  void (*proc)();
  int  flags;
} mainmenuitems[] = {
  { "ai",   "Add Image",      menusel_addimage     , 0  },
  { "li",   "List Images",    menusel_listimages   , 0 },
  { "si",   "Show Image",     menusel_showimage    , 0 },
  { "cl",   "Create Link",    menusel_createlink   , 0 },
  { "dl",   "Delete Link",    menusel_deletelink   , 0 },
  { "r",    "Render",         menusel_render       , 0 },
  { "rq",   "Render Quick",   menusel_render       , RENDER_QUICK },
  { "rtf",  "Render To File", menusel_render       , RENDER_TOFILE },
  { "load", "Load Images and Links", menusel_load  , 0 },
  { "save", "Save Images and Links", menusel_save  , 0 },
  { "q",    "Quit",           menusel_quit         , 0 },
  { NULL , NULL,  NULL, 0 }
};

void
wait_on_imagedisplayer( ImageDisplayer *id)
{
  int x, y;

  printf("Click in the window to continue...\n");
  imagedisplayer_get_buttonpress(id, &x, &y);
}

void
main(int argc, char *argv[])
{
  int argctr;

  printf("\nWelcome to the Projective Transform Tool\n");
  printf("Written February 1996 by Erik Nygren for 6.866 Final Project\n\n");

  allimages = imageinfolist_create();
  imagedisplayer_init();

  for (argctr = 1; argctr < argc; argctr++)
    {
      ImageInfo *imi;

      imi = imageinfo_create_fromfilename(argv[argctr]);
      if (NULL == imi)
	fprintf(stderr, "Couldn't load image %s.\n", argv[argctr]);
      else
	{
	  imageinfolist_addtolist(allimages, imi);
	}
    }
    
  /* Main menu */
  while (1)
    {
      int ctr, found;
      char choice[256];

      /* Print the options */
      printf("\nMenu options:\n");
      for (ctr=0; mainmenuitems[ctr].abbrev; ctr++)
	printf("  [%-4s]\t%s\n", mainmenuitems[ctr].abbrev, 
	       mainmenuitems[ctr].itemname);
      printf("> ");
      scanf("%s", choice);
      printf("\n");

      /* Dispatch */
      found = 0;
      for (ctr=0; mainmenuitems[ctr].abbrev; ctr++)
	if (0 == strcmp(mainmenuitems[ctr].abbrev, choice))
	  {
	    found = 1;
	    printf("\n");
	    (mainmenuitems[ctr].proc)(mainmenuitems[ctr].flags);
	  }
      if (!found)
	printf("Invalid menu selection.\n\n");
    }
}



void
menusel_unimplemented(int flags)
{
  printf("Menu option not implemented.\n\n");
}

void
menusel_quit(int flags)
{
  printf("Bye!\n\n");
  exit(0);
}

void
menusel_createlink(int flags)
{
  ImageInfo *imiA, *imiB;
  int npts;

  printf("Create Link\n");
  printf("-----------\n");
  imiA = imageinfo_promptforimage(allimages, 
				  "Select image A");
  if (!imiA) return;
  imiB = imageinfo_promptforimage(allimages, 
				  "Select image B");
  if (!imiB) return;
  printf("How many points pairs do you want to select? (min 5) > ");
  scanf("%d", &npts);

  imageinfo_makelinkage(imiA, imiB, npts);
}

void
menusel_deletelink(int flags)
{
  ImageInfo *imiA, *imiB;

  printf("Delete Link\n");
  printf("-----------\n");
  imiA = imageinfo_promptforimage(allimages, 
				  "Select one side of the link");
  if (!imiA) return;
  imiB = imageinfo_promptforimage(allimages, 
				  "Select other side of the link");
  if (!imiB) return;

  imageinfo_deletelink(imiA, imiB);
}


void
menusel_listimages(int flags)
{
  imageinfolist_print(allimages);
}

void
menusel_showimage(int flags)
{
  ImageInfo *imi;
  ImageDisplayer *id;

  imi = imageinfo_promptforimage(allimages, 
				 "Select image to display");
  if (!imi) return;
  id = imagedisplayer_create(imi->im->xres, imi->im->yres, imi->name);
  imagedisplayer_putimage(id, imi->im);
  wait_on_imagedisplayer(id);
  imagedisplayer_destroy(id);
}

void
menusel_load(int flags)
{
  char filename[256];

  printf("Load Images And Links\n");
  printf("---------------------\n");
  printf("Enter filename of images and links to load > ");
  scanf("%s",filename);
  printf("Loading...\n");
  imageinfolist_load(allimages, filename);
  printf("Done.\n\n");
}

void
menusel_save(int flags)
{
  char filename[256];

  printf("Save Images And Links\n");
  printf("---------------------\n");
  printf("Enter name of file to save images and links to > ");
  scanf("%s",filename);
  printf("Saving...\n");
  imageinfolist_saveall(allimages, filename);
  printf("Done.\n\n");
}


void
menusel_addimage(int flags)
{
  char      filename[256];
  ImageInfo *imi;

  printf("Add Image\n");
  printf("---------\n");
  printf("Enter filename of image to load > ");
  scanf("%s",filename);

  imi = imageinfo_create_fromfilename(filename);
  if (NULL == imi)
    fprintf(stderr, "Couldn't load image %s.\n", filename);
  else
    {
      imageinfolist_addtolist(allimages, imi);
    }

  printf("Done.\n\n");
}

void
menusel_render(int flags)
{
  char filename[256];
  ImageInfo *centerimi;
  Image     *renderedim;
  ImageDisplayer *id;
  int xres, yres;
  int fd;

  if (flags & RENDER_TOFILE)
    {
      printf("Render To File\n");
      printf("-------------\n");
      printf("Enter filename to render to > ");
      scanf("%s",filename);
    }

  centerimi = imageinfo_promptforimage(allimages, 
				  "Select center image");
  if (!centerimi) return;

  printf("Horizontal resolution? > ");
  scanf("%d", &xres);
  printf("Vertical resolution? > ");
  scanf("%d", &yres);
  if (xres < 5 | yres < 5)
    {
      printf("Invalid resolution.\n");
      return;
    }

  renderedim = imageinfo_rendertree(allimages, centerimi, xres, yres, 1,
				    (flags&RENDER_QUICK)?1:0);

  id = imagedisplayer_create(renderedim->xres, renderedim->yres, 
			     "Rendered Image");
  imagedisplayer_putimage(id, renderedim);
  wait_on_imagedisplayer(id);
  imagedisplayer_destroy(id);

  if (flags & RENDER_TOFILE)
    {
      if (-1 == (fd = open(filename, O_WRONLY | O_CREAT | O_TRUNC, 0644)))
	{ fprintf(stderr,"Unable to open file %s to save to.\n", filename);
	  return; }
      image_writepnm(renderedim, fd);
      close(fd);
    }

  image_destroy(renderedim);
}

